@@ -1,4 +1,4 @@
-/* NetHack 3.6	winprocs.h	$NHDT-Date: 1433806582 2015/06/08 23:36:22 $  $NHDT-Branch: master $:$NHDT-Revision: 1.36 $ */
+/* NetHack 3.6	winprocs.h	$NHDT-Date: 1502141230 2017/08/07 21:27:10 $  $NHDT-Branch: NetHack-3.6.0 $:$NHDT-Revision: 1.38 $ */
 /* Copyright (c) David Cohrs, 1992				  */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -13,8 +13,7 @@ struct window_procs {
                            * not start with '-'.  Names starting with
                            * '+' are reserved for processors. */
     unsigned long wincap; /* window port capability options supported */
-    unsigned long
-        wincap2; /* additional window port capability options supported */
+    unsigned long wincap2; /* additional window port capability options */
     void FDECL((*win_init_nhwindows), (int *, char **));
     void NDECL((*win_player_selection));
     void NDECL((*win_askname));
@@ -74,16 +73,11 @@ struct window_procs {
     void FDECL((*win_preference_update), (const char *));
     char *FDECL((*win_getmsghistory), (BOOLEAN_P));
     void FDECL((*win_putmsghistory), (const char *, BOOLEAN_P));
-#ifdef STATUS_VIA_WINDOWPORT
     void NDECL((*win_status_init));
     void NDECL((*win_status_finish));
     void FDECL((*win_status_enablefield),
                (int, const char *, const char *, BOOLEAN_P));
-    void FDECL((*win_status_update), (int, genericptr_t, int, int));
-#ifdef STATUS_HILITES
-    void FDECL((*win_status_threshold), (int, int, anything, int, int, int));
-#endif
-#endif
+    void FDECL((*win_status_update), (int, genericptr_t, int, int, int, unsigned long *));
     boolean NDECL((*win_can_suspend));
 };
 
@@ -161,107 +155,84 @@ extern
 #define preference_update (*windowprocs.win_preference_update)
 #define getmsghistory (*windowprocs.win_getmsghistory)
 #define putmsghistory (*windowprocs.win_putmsghistory)
-#ifdef STATUS_VIA_WINDOWPORT
 /* there is a status_initialize() in botl.c,
  * which calls win_status_init() directly; same with status_finish.
  */
 #define status_enablefield (*windowprocs.win_status_enablefield)
 #define status_update (*windowprocs.win_status_update)
-#ifdef STATUS_HILITES
-#define status_threshold (*windowprocs.win_status_threshold)
-#endif
-#endif
 
 /*
  * WINCAP
  * Window port preference capability bits.
  * Some day this might be better in its own wincap.h file.
  */
-#define WC_COLOR 0x01L      /* 01 Port can display things in color       */
-#define WC_HILITE_PET 0x02L /* 02 supports hilite pet                    */
-#define WC_ASCII_MAP 0x04L  /* 03 supports an ascii map                  */
-#define WC_TILED_MAP 0x08L  /* 04 supports a tiled map                   */
-#define WC_PRELOAD_TILES 0x10L /* 05 supports pre-loading tiles */
-#define WC_TILE_WIDTH 0x20L  /* 06 prefer this width of tile              */
-#define WC_TILE_HEIGHT 0x40L /* 07 prefer this height of tile             */
-#define WC_TILE_FILE 0x80L   /* 08 alternative tile file name             */
-#define WC_INVERSE 0x100L    /* 09 Port supports inverse video            */
-#define WC_ALIGN_MESSAGE \
-    0x200L /* 10 supports message alignmt top|b|l|r     */
-#define WC_ALIGN_STATUS 0x400L /* 11 supports status alignmt top|b|l|r */
-#define WC_VARY_MSGCOUNT \
-    0x800L                  /* 12 supports varying message window        */
-#define WC_FONT_MAP 0x1000L /* 13 supports specification of map win font */
-#define WC_FONT_MESSAGE \
-    0x2000L                    /* 14 supports specification of msg win font */
-#define WC_FONT_STATUS 0x4000L /* 15 supports specification of sts win font \
-                                  */
-#define WC_FONT_MENU 0x8000L   /* 16 supports specification of mnu win font */
-#define WC_FONT_TEXT 0x10000L  /* 17 supports specification of txt win font \
-                                  */
-#define WC_FONTSIZ_MAP \
-    0x20000L /* 18 supports specification of map win font */
-#define WC_FONTSIZ_MESSAGE \
-    0x40000L /* 19 supports specification of msg win font */
-#define WC_FONTSIZ_STATUS \
-    0x80000L /* 20 supports specification of sts win font */
-#define WC_FONTSIZ_MENU \
-    0x100000L /* 21 supports specification of mnu win font */
-#define WC_FONTSIZ_TEXT \
-    0x200000L /* 22 supports specification of txt win font */
-#define WC_SCROLL_MARGIN \
-    0x400000L /* 23 supports setting scroll margin for map */
-#define WC_SPLASH_SCREEN \
-    0x800000L /* 24 supports display of splash screen      */
-#define WC_POPUP_DIALOG \
-    0x1000000L /* 25 supports queries in pop dialogs        */
-#define WC_SCROLL_AMOUNT \
-    0x2000000L /* 26 scroll this amount at scroll margin    */
-#define WC_EIGHT_BIT_IN \
-    0x4000000L /* 27 8-bit character input                  */
-#define WC_PERM_INVENT \
-    0x8000000L /* 28 8-bit character input                  */
-#define WC_MAP_MODE \
-    0x10000000L /* 29 map_mode option                        */
-#define WC_WINDOWCOLORS \
-    0x20000000L /* 30 background color for message window    */
-#define WC_PLAYER_SELECTION \
-    0x40000000L /* 31 background color for message window    */
+/* clang-format off */
+#define WC_COLOR         0x00000001L /* 01 Port can display things in color  */
+#define WC_HILITE_PET    0x00000002L /* 02 supports hilite pet               */
+#define WC_ASCII_MAP     0x00000004L /* 03 supports an ascii map             */
+#define WC_TILED_MAP     0x00000008L /* 04 supports a tiled map              */
+#define WC_PRELOAD_TILES 0x00000010L /* 05 supports pre-loading tiles        */
+#define WC_TILE_WIDTH    0x00000020L /* 06 prefer this width of tile         */
+#define WC_TILE_HEIGHT   0x00000040L /* 07 prefer this height of tile        */
+#define WC_TILE_FILE     0x00000080L /* 08 alternative tile file name        */
+#define WC_INVERSE       0x00000100L /* 09 Port supports inverse video       */
+#define WC_ALIGN_MESSAGE 0x00000200L /* 10 supports mesg alignment top|b|l|r */
+#define WC_ALIGN_STATUS  0x00000400L /* 11 supports status alignmt top|b|l|r */
+#define WC_VARY_MSGCOUNT 0x00000800L /* 12 supports varying message window   */
+#define WC_FONT_MAP      0x00001000L /* 13 supports spec of map window font  */
+#define WC_FONT_MESSAGE  0x00002000L /* 14 supports spec of message font     */
+#define WC_FONT_STATUS   0x00004000L /* 15 supports spec of status font      */
+#define WC_FONT_MENU     0x00008000L /* 16 supports spec of menu font        */
+#define WC_FONT_TEXT     0x00010000L /* 17 supports spec of text window font */
+#define WC_FONTSIZ_MAP   0x00020000L /* 18 supports spec of map font size    */
+#define WC_FONTSIZ_MESSAGE 0x040000L /* 19 supports spec of mesg font size   */
+#define WC_FONTSIZ_STATUS 0x0080000L /* 20 supports spec of status font size */
+#define WC_FONTSIZ_MENU  0x00100000L /* 21 supports spec of menu font size   */
+#define WC_FONTSIZ_TEXT  0x00200000L /* 22 supports spec of text font size   */
+#define WC_SCROLL_MARGIN 0x00400000L /* 23 supports setting map scroll marg  */
+#define WC_SPLASH_SCREEN 0x00800000L /* 24 supports display of splash screen */
+#define WC_POPUP_DIALOG  0x01000000L /* 25 supports queries in popup dialogs */
+#define WC_SCROLL_AMOUNT 0x02000000L /* 26 scroll this amount at scroll marg */
+#define WC_EIGHT_BIT_IN  0x04000000L /* 27 8-bit character input             */
+#define WC_PERM_INVENT   0x08000000L /* 28 supports persistent inventory win */
+#define WC_MAP_MODE      0x10000000L /* 29 map_mode option                   */
+#define WC_WINDOWCOLORS  0x20000000L /* 30 background color for mesg window  */
+#define WC_PLAYER_SELECTION 0x40000000L /* 31 supports player selection      */
 #ifdef NHSTDC
-#define WC_MOUSE_SUPPORT \
-    0x80000000UL /* 32 mouse support                          */
+#define WC_MOUSE_SUPPORT 0x80000000UL /* 32 mouse support                    */
 #else
-#define WC_MOUSE_SUPPORT \
-    0x80000000L /* 32 mouse support                          */
+#define WC_MOUSE_SUPPORT 0x80000000L /* 32 mouse support                     */
 #endif
-/* no free bits */
+                                     /* no free bits */
 
-#define WC2_FULLSCREEN 0x01L /* 01 display full screen                    */
-#define WC2_SOFTKEYBOARD 0x02L /* 02 software keyboard */
-#define WC2_WRAPTEXT 0x04L /* 03 wrap long lines of text                */
-#define WC2_HILITE_STATUS \
-    0x08L /* 04 hilite fields in status                */
-#define WC2_SELECTSAVED 0x10L /* 05 saved game selection menu */
-#define WC2_DARKGRAY 0x20L /* 06 use bold black for black glyphs        */
-                           /* 26 free bits */
+#define WC2_FULLSCREEN    0x0001L /* 01 display full screen             */
+#define WC2_SOFTKEYBOARD  0x0002L /* 02 software keyboard               */
+#define WC2_WRAPTEXT      0x0004L /* 03 wrap long lines of text         */
+#define WC2_HILITE_STATUS 0x0008L /* 04 hilite fields in status         */
+#define WC2_SELECTSAVED   0x0010L /* 05 saved game selection menu       */
+#define WC2_DARKGRAY      0x0020L /* 06 use bold black for black glyphs */
+#define WC2_HITPOINTBAR   0x0040L /* 07 show bar representing hit points */
+#define WC2_FLUSH_STATUS  0x0080L /* 08 call status_update(BL_FLUSH)
+                                        after updating status window fields */
+                                  /* 24 free bits */
 
-#define ALIGN_LEFT 1
-#define ALIGN_RIGHT 2
-#define ALIGN_TOP 3
+#define ALIGN_LEFT   1
+#define ALIGN_RIGHT  2
+#define ALIGN_TOP    3
 #define ALIGN_BOTTOM 4
 
 /* player_selection */
-#define VIA_DIALOG 0
+#define VIA_DIALOG  0
 #define VIA_PROMPTS 1
 
 /* map_mode settings - deprecated */
-#define MAP_MODE_TILES 0
-#define MAP_MODE_ASCII4x6 1
-#define MAP_MODE_ASCII6x8 2
-#define MAP_MODE_ASCII8x8 3
-#define MAP_MODE_ASCII16x8 4
-#define MAP_MODE_ASCII7x12 5
-#define MAP_MODE_ASCII8x12 6
+#define MAP_MODE_TILES      0
+#define MAP_MODE_ASCII4x6   1
+#define MAP_MODE_ASCII6x8   2
+#define MAP_MODE_ASCII8x8   3
+#define MAP_MODE_ASCII16x8  4
+#define MAP_MODE_ASCII7x12  5
+#define MAP_MODE_ASCII8x12  6
 #define MAP_MODE_ASCII16x12 7
 #define MAP_MODE_ASCII12x16 8
 #define MAP_MODE_ASCII10x18 9
@@ -269,13 +240,13 @@ extern
 #define MAP_MODE_TILES_FIT_TO_SCREEN 11
 
 #if 0
-#define WC_SND_SOUND 0x01L   /* 01 Port has some sound capabilities       */
-#define WC_SND_SPEAKER 0x02L /* 02 Sound supported via built-in speaker   */
-#define WC_SND_STEREO 0x04L  /* 03 Stereo sound supported                 */
-#define WC_SND_RAW 0x08L     /* 04 Raw sound supported                    */
-#define WC_SND_WAVE 0x10L    /* 05 Wave support                           */
-#define WC_SND_MIDI 0x20L /* 06 Midi support                           */
-					/* 26 free bits */
+#define WC_SND_SOUND   0x0001L /* 01 Port has some sound capabilities     */
+#define WC_SND_SPEAKER 0x0002L /* 02 Sound supported via built-in speaker */
+#define WC_SND_STEREO  0x0004L /* 03 Stereo sound supported               */
+#define WC_SND_RAW     0x0008L /* 04 Raw sound supported                  */
+#define WC_SND_WAVE    0x0010L /* 05 Wave support                         */
+#define WC_SND_MIDI    0x0020L /* 06 Midi support                         */
+                               /* 26 free bits */
 #endif
 
 struct wc_Opt {
@@ -284,17 +255,17 @@ struct wc_Opt {
 };
 
 /* role selection by player_selection(); this ought to be in the core... */
-#define RS_NAME 0
-#define RS_ROLE 1
-#define RS_RACE 2
-#define RS_GENDER 3
+#define RS_NAME    0
+#define RS_ROLE    1
+#define RS_RACE    2
+#define RS_GENDER  3
 #define RS_ALGNMNT 4
-#define RS_filter 5
+#define RS_filter  5
 #define RS_menu_arg(x) (ROLE_RANDOM - ((x) + 1)) /* 0..5 -> -3..-8 */
 
 /* Choose_windows() may be called multiple times; these constants tell the
  * init function whether the window system is coming or going. */
-#define WININIT 0
+#define WININIT      0
 #define WININIT_UNDO 1
 
 #ifdef WINCHAIN
@@ -310,7 +281,9 @@ struct wc_Opt {
         nextdata is the Xprivate* for the next link in the chain
 */
 #define WINCHAIN_ALLOC 0
-#define WINCHAIN_INIT 1
+#define WINCHAIN_INIT  1
+
+/* clang-format on */
 
 #define CARGS void *
 
@@ -321,8 +294,7 @@ struct chain_procs {
                            * not start with '-'.  Names starting with
                            * '+' are reserved for processors. */
     unsigned long wincap; /* window port capability options supported */
-    unsigned long
-        wincap2; /* additional window port capability options supported */
+    unsigned long wincap2; /* additional window port capability options */
     void FDECL((*win_init_nhwindows), (CARGS, int *, char **));
     void FDECL((*win_player_selection), (CARGS));
     void FDECL((*win_askname), (CARGS));
@@ -383,17 +355,11 @@ struct chain_procs {
     void FDECL((*win_preference_update), (CARGS, const char *));
     char *FDECL((*win_getmsghistory), (CARGS, BOOLEAN_P));
     void FDECL((*win_putmsghistory), (CARGS, const char *, BOOLEAN_P));
-#ifdef STATUS_VIA_WINDOWPORT
     void FDECL((*win_status_init), (CARGS));
     void FDECL((*win_status_finish), (CARGS));
     void FDECL((*win_status_enablefield),
                (CARGS, int, const char *, const char *, BOOLEAN_P));
-    void FDECL((*win_status_update), (CARGS, int, genericptr_t, int, int));
-#ifdef STATUS_HILITES
-    void FDECL((*win_status_threshold),
-               (CARGS, int, int, anything, int, int, int));
-#endif
-#endif
+    void FDECL((*win_status_update), (CARGS, int, genericptr_t, int, int, int, unsigned long *));
     boolean FDECL((*win_can_suspend), (CARGS));
 };
 #endif /* WINCHAIN */
