@@ -1,5 +1,6 @@
-/* NetHack 3.6	restore.c	$NHDT-Date: 1446892455 2015/11/07 10:34:15 $  $NHDT-Branch: master $:$NHDT-Revision: 1.101 $ */
+/* NetHack 3.6	restore.c	$NHDT-Date: 1451082255 2015/12/25 22:24:15 $  $NHDT-Branch: NetHack-3.6.0 $:$NHDT-Revision: 1.103 $ */
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
+/*-Copyright (c) Michael Allison, 2009. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
@@ -531,7 +532,7 @@ unsigned int *stuckid, *steedid;
             return FALSE;
     }
     mread(fd, (genericptr_t) &context, sizeof(struct context_info));
-    if (context.warntype.speciesidx)
+    if (context.warntype.speciesidx >= LOW_PM)
         context.warntype.species = &mons[context.warntype.speciesidx];
 
     /* we want to be able to revert to command line/environment/config
@@ -568,13 +569,10 @@ unsigned int *stuckid, *steedid;
     foo = time_from_yyyymmddhhmmss(timebuf);
 
     ReadTimebuf(ubirthday);
-    mread(fd, &urealtime.realtime, sizeof(urealtime.realtime));
-    ReadTimebuf(urealtime.restored);
-#if defined(BSD) && !defined(POSIX_TYPES)
-    (void) time((long *) &urealtime.restored);
-#else
-    (void) time(&urealtime.restored);
-#endif
+    mread(fd, &urealtime.realtime, sizeof urealtime.realtime);
+    ReadTimebuf(urealtime.start_timing); /** [not used] **/
+    /* current time is the time to use for next urealtime.realtime update */
+    urealtime.start_timing = getnow();
 
     set_uasmon();
 #ifdef CLIPPING
@@ -747,7 +745,7 @@ xchar ltmp;
         }
 #endif /* ?AMIGA */
         pline("Be seeing you...");
-        terminate(EXIT_SUCCESS);
+        nh_terminate(EXIT_SUCCESS);
     }
 #endif /* MFLOPPY */
     bufon(nfd);
