@@ -1,5 +1,6 @@
-/* NetHack 3.6	teleport.c	$NHDT-Date: 1446887535 2015/11/07 09:12:15 $  $NHDT-Branch: master $:$NHDT-Revision: 1.62 $ */
+/* NetHack 3.6	teleport.c	$NHDT-Date: 1523306912 2018/04/09 20:48:32 $  $NHDT-Branch: NetHack-3.6.0 $:$NHDT-Revision: 1.73 $ */
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
+/*-Copyright (c) Robert Patrick Rankin, 2011. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
@@ -128,7 +129,9 @@ unsigned entflags;
         /* default to player's original monster type */
         mdat = &mons[u.umonster];
     }
-    fakemon.data = mdat; /* set up for goodpos */
+    fakemon = zeromonst;
+    set_mon_data(&fakemon, mdat, -1); /* set up for goodpos */
+
     good_ptr = good;
     range = 1;
     /*
@@ -196,6 +199,8 @@ STATIC_OVL boolean
 tele_jump_ok(x1, y1, x2, y2)
 int x1, y1, x2, y2;
 {
+    if (!isok(x2, y2))
+        return FALSE;
     if (dndest.nlx > 0) {
         /* if inside a restricted region, can't teleport outside */
         if (within_bounded_area(x1, y1, dndest.nlx, dndest.nly, dndest.nhx,
@@ -252,7 +257,7 @@ boolean allow_drag;
         /* unearth it */
         buried_ball_to_punishment();
     }
-    ball_active = (Punished && uball->where != OBJ_FREE),
+    ball_active = (Punished && uball->where != OBJ_FREE);
     ball_still_in_range = FALSE;
 
     /* If they have to move the ball, then drag if allow_drag is true;
@@ -605,12 +610,22 @@ level_tele()
 
         Strcpy(qbuf, "To what level do you want to teleport?");
         do {
+            if (iflags.menu_requested) {
+                /* wizard mode 'm ^V' skips prompting on first pass
+                   (note: level Tport via menu won't have any second pass) */
+                iflags.menu_requested = FALSE;
+                if (wizard)
+                    goto levTport_menu;
+            }
             if (++trycnt == 2) {
                 if (wizard)
-                    Strcat(qbuf, " [type a number or ? for a menu]");
+                    Strcat(qbuf, " [type a number, name, or ? for a menu]");
                 else
-                    Strcat(qbuf, " [type a number]");
+                    Strcat(qbuf, " [type a number or name]");
             }
+            *buf = '\0'; /* EDIT_GETLIN: if we're on second or later pass,
+                            the previous input was invalid so don't use it
+                            as getlin()'s preloaded default answer */
             getlin(qbuf, buf);
             if (!strcmp(buf, "\033")) { /* cancelled */
                 if (Confusion && rnl(5)) {
@@ -625,9 +640,12 @@ level_tele()
                 goto random_levtport;
             }
             if (wizard && !strcmp(buf, "?")) {
-                schar destlev = 0;
-                xchar destdnum = 0;
+                schar destlev;
+                xchar destdnum;
 
+            levTport_menu:
+                destlev = 0;
+                destdnum = 0;
                 newlev = (int) print_dungeon(TRUE, &destlev, &destdnum);
                 if (!newlev)
                     return;
@@ -870,12 +888,20 @@ struct trap *trap;
 }
 
 void
-level_tele_trap(trap)
+level_tele_trap(trap, trflags)
 struct trap *trap;
+unsigned trflags;
 {
-    You("%s onto a level teleport trap!",
-        Levitation ? (const char *) "float"
-                   : locomotion(youmonst.data, "step"));
+    char verbbuf[BUFSZ];
+
+    if ((trflags & VIASITTING) != 0)
+        Strcpy(verbbuf, "trigger"); /* follows "You sit down." */
+    else
+        Sprintf(verbbuf, "%s onto",
+                Levitation ? (const char *) "float"
+                           : locomotion(youmonst.data, "step"));
+    You("%s a level teleport trap!", verbbuf);
+
     if (Antimagic) {
         shieldeff(u.ux, u.uy);
     }
@@ -954,11 +980,9 @@ struct monst *mtmp;
  *
  * Pulls a monster from its current position and places a monster at
  * a new x and y.  If oldx is 0, then the monster was not in the
- * levels.monsters
- * array.  However, if oldx is 0, oldy may still have a value because mtmp is
- * a
- * migrating_mon.  Worm tails are always placed randomly around the head of
- * the worm.
+ * levels.monsters array.  However, if oldx is 0, oldy may still have
+ * a value because mtmp is a migrating_mon.  Worm tails are always
+ * placed randomly around the head of the worm.
  */
 void
 rloc_to(mtmp, x, y)
@@ -972,14 +996,15 @@ register int x, y;
         return;
 
     if (oldx) { /* "pick up" monster */
-        if (mtmp->wormno)
+        if (mtmp->wormno) {
             remove_worm(mtmp);
-        else {
+        } else {
             remove_monster(oldx, oldy);
             newsym(oldx, oldy); /* update old location */
         }
     }
 
+    memset(mtmp->mtrack, 0, sizeof mtmp->mtrack);
     place_monster(mtmp, x, y); /* put monster down */
     update_monster_region(mtmp);
 
@@ -1126,8 +1151,7 @@ struct trap *trap;
 boolean force_it;
 int in_sight;
 {
-    int tt = trap->ttyp;
-    struct permonst *mptr = mtmp->data;
+    int tt = (trap ? trap->ttyp : NO_TRAP);
 
     if (mtmp == u.ustuck) /* probably a vortex */
         return 0;         /* temporary? kludge */
@@ -1148,8 +1172,8 @@ int in_sight;
             }
         } else if (tt == MAGIC_PORTAL) {
             if (In_endgame(&u.uz)
-                && (mon_has_amulet(mtmp) || is_home_elemental(mptr))) {
-                if (in_sight && mptr->mlet != S_ELEMENTAL) {
+                && (mon_has_amulet(mtmp) || is_home_elemental(mtmp->data))) {
+                if (in_sight && mtmp->data->mlet != S_ELEMENTAL) {
                     pline("%s seems to shimmer for a moment.", Monnam(mtmp));
                     seetrap(trap);
                 }
@@ -1158,27 +1182,44 @@ int in_sight;
                 assign_level(&tolevel, &trap->dst);
                 migrate_typ = MIGR_PORTAL;
             }
-        } else { /* (tt == LEVEL_TELEP) */
+        } else if (tt == LEVEL_TELEP || tt == NO_TRAP) {
             int nlev;
 
-            if (mon_has_amulet(mtmp) || In_endgame(&u.uz)) {
+            if (mon_has_amulet(mtmp) || In_endgame(&u.uz)
+                /* NO_TRAP is used when forcing a monster off the level;
+                   onscary(0,0,) is true for the Wizard, Riders, lawful
+                   minions, Angels of any alignment, shopkeeper or priest
+                   currently inside his or her own special room */
+                || (tt == NO_TRAP && onscary(0, 0, mtmp))) {
                 if (in_sight)
                     pline("%s seems very disoriented for a moment.",
                           Monnam(mtmp));
                 return 0;
             }
-            nlev = random_teleport_level();
-            if (nlev == depth(&u.uz)) {
-                if (in_sight)
-                    pline("%s shudders for a moment.", Monnam(mtmp));
-                return 0;
+            if (tt == NO_TRAP) {
+                /* creature is being forced off the level to make room;
+                   it will try to return to this level (at a random spot
+                   rather than its current one) if the level is left by
+                   the hero and then revisited */
+                assign_level(&tolevel, &u.uz);
+            } else {
+                nlev = random_teleport_level();
+                if (nlev == depth(&u.uz)) {
+                    if (in_sight)
+                        pline("%s shudders for a moment.", Monnam(mtmp));
+                    return 0;
+                }
+                get_level(&tolevel, nlev);
             }
-            get_level(&tolevel, nlev);
+        } else {
+            impossible("mlevel_tele_trap: unexpected trap type (%d)", tt);
+            return 0;
         }
 
         if (in_sight) {
             pline("Suddenly, %s disappears out of sight.", mon_nam(mtmp));
-            seetrap(trap);
+            if (trap)
+                seetrap(trap);
         }
         migrate_to_level(mtmp, ledger_no(&tolevel), migrate_typ, (coord *) 0);
         return 3; /* no longer on this level */
