@@ -1,5 +1,6 @@
-/* NetHack 3.6	vault.c	$NHDT-Date: 1446078792 2015/10/29 00:33:12 $  $NHDT-Branch: master $:$NHDT-Revision: 1.39 $ */
+/* NetHack 3.6	vault.c	$NHDT-Date: 1452132199 2016/01/07 02:03:19 $  $NHDT-Branch: NetHack-3.6.0 $:$NHDT-Revision: 1.42 $ */
 /* Copyright (c) Stichting Mathematisch Centrum, Amsterdam, 1985. */
+/*-Copyright (c) Robert Patrick Rankin, 2011. */
 /* NetHack may be freely redistributed.  See license for details. */
 
 #include "hack.h"
@@ -191,6 +192,13 @@ findgd()
     return (struct monst *) 0;
 }
 
+void
+vault_summon_gd()
+{
+    if (vault_occupied(u.urooms) && !findgd())
+        u.uinvault = (VAULT_GUARD_TIME - 1);
+}
+
 char
 vault_occupied(array)
 char *array;
@@ -221,8 +229,9 @@ invault()
     vaultroom -= ROOMOFFSET;
 
     guard = findgd();
-    if (++u.uinvault % 30 == 0 && !guard) { /* if time ok and no guard now. */
-        char buf[BUFSZ];
+    if (++u.uinvault % VAULT_GUARD_TIME == 0 && !guard) {
+        /* if time ok and no guard now. */
+        char buf[BUFSZ] = DUMMY;
         register int x, y, dd, gx, gy;
         int lx = 0, ly = 0;
         long umoney;
@@ -342,7 +351,7 @@ invault()
         newsym(guard->mx, guard->my);
         if (u.uswallow) {
             /* can't interrogate hero, don't interrogate engulfer */
-            verbalize("What's going on here?");
+            if (!Deaf) verbalize("What's going on here?");
             if (gsensed)
                 pline_The("other presence vanishes.");
             mongone(guard);
@@ -351,8 +360,8 @@ invault()
         if (youmonst.m_ap_type == M_AP_OBJECT || u.uundetected) {
             if (youmonst.m_ap_type == M_AP_OBJECT
                 && youmonst.mappearance != GOLD_PIECE)
-                verbalize("Hey! Who left that %s in here?",
-                          mimic_obj_name(&youmonst));
+                if (!Deaf) verbalize("Hey! Who left that %s in here?",
+                                    mimic_obj_name(&youmonst));
             /* You're mimicking some object or you're hidden. */
             pline("Puzzled, %s turns around and leaves.", mhe(guard));
             mongone(guard);
@@ -362,7 +371,10 @@ invault()
             /* [we ought to record whether this this message has already
                been given in order to vary it upon repeat visits, but
                discarding the monster and its egd data renders that hard] */
-            verbalize("I'll be back when you're ready to speak to me!");
+            if (Deaf)
+                pline("%s huffs and turns to leave.", noit_Monnam(guard));
+            else
+                verbalize("I'll be back when you're ready to speak to me!");
             mongone(guard);
             return;
         }
@@ -374,9 +386,10 @@ invault()
         }
         trycount = 5;
         do {
-            getlin("\"Hello stranger, who are you?\" -", buf);
+            getlin(Deaf ? "You are required to supply your name. -"
+                        : "\"Hello stranger, who are you?\" -", buf);
             (void) mungspaces(buf);
-        } while (!letter(buf[0]) && --trycount > 0);
+        } while (!buf[0] && --trycount > 0);
 
         if (u.ualign.type == A_LAWFUL
             /* ignore trailing text, in case player includes rank */
@@ -387,12 +400,24 @@ invault()
         if (!strcmpi(buf, "Croesus") || !strcmpi(buf, "Kroisos")
             || !strcmpi(buf, "Creosote")) {
             if (!mvitals[PM_CROESUS].died) {
-                verbalize(
-                    "Oh, yes, of course.  Sorry to have disturbed you.");
+                if (Deaf) {
+                    if (!Blind)
+                        pline("%s waves goodbye.", noit_Monnam(guard));
+                } else {
+                    verbalize(
+                         "Oh, yes, of course.  Sorry to have disturbed you.");
+                }
                 mongone(guard);
             } else {
-                setmangry(guard);
-                verbalize("Back from the dead, are you?  I'll remedy that!");
+                setmangry(guard, FALSE);
+                if (Deaf) {
+                   if (!Blind)
+                        pline("%s mouths something and looks very angry!",
+                              noit_Monnam(guard));
+                } else {
+                   verbalize(
+                           "Back from the dead, are you?  I'll remedy that!");
+                }
                 /* don't want guard to waste next turn wielding a weapon */
                 if (!MON_WEP(guard)) {
                     guard->weapon_check = NEED_HTH_WEAPON;
@@ -401,18 +426,38 @@ invault()
             }
             return;
         }
-        verbalize("I don't know you.");
+        if (Deaf)
+            pline("%s doesn't %srecognize you.", noit_Monnam(guard),
+                    (Blind) ? "" : "appear to ");
+        else
+            verbalize("I don't know you.");
         umoney = money_cnt(invent);
-        if (Deaf) {
-            ;
-        } else if (!umoney && !hidden_gold()) {
-            verbalize("Please follow me.");
+        if (!umoney && !hidden_gold()) {
+            if (Deaf)
+                pline("%s stomps%s.", noit_Monnam(guard),
+                      (Blind) ? "" : " and beckons");
+            else
+                verbalize("Please follow me.");
         } else {
-            if (!umoney)
-                verbalize("You have hidden gold.");
-            verbalize(
-                "Most likely all your gold was stolen from this vault.");
-            verbalize("Please drop that gold and follow me.");
+            if (!umoney) {
+                if (Deaf) {
+                    if (!Blind)
+                        pline("%s glares at you%s.", noit_Monnam(guard),
+                              invent ? "r stuff" : "");
+                } else {
+                   verbalize("You have hidden gold.");
+                }
+            }
+            if (Deaf) {
+                if (!Blind)
+                    pline(
+                       "%s holds out %s palm and beckons with %s other hand.",
+                          noit_Monnam(guard), mhis(guard), mhis(guard));
+            } else {
+                verbalize(
+                    "Most likely all your gold was stolen from this vault.");
+                verbalize("Please drop that gold and follow me.");
+            }
         }
         EGD(guard)->gdx = gx;
         EGD(guard)->gdy = gy;
@@ -578,8 +623,9 @@ register struct monst *grd;
         return -1; /* teleported guard - treat as monster */
 
     if (egrd->witness) {
-        verbalize("How dare you %s that gold, scoundrel!",
-                  (egrd->witness & GD_EATGOLD) ? "consume" : "destroy");
+        if (!Deaf)
+            verbalize("How dare you %s that gold, scoundrel!",
+                      (egrd->witness & GD_EATGOLD) ? "consume" : "destroy");
         egrd->witness = 0;
         grd->mpeaceful = 0;
         return -1;
@@ -655,12 +701,22 @@ register struct monst *grd;
             }
             if (egrd->warncnt < 6) {
                 egrd->warncnt = 6;
-                if (!Deaf)
+                if (Deaf) {
+                    if (!Blind)
+                        pline("%s holds out %s palm demandingly!",
+                              noit_Monnam(grd), mhis(grd));
+                } else {
                     verbalize("Drop all your gold, scoundrel!");
+                }
                 return 0;
             } else {
-                if (!Deaf)
+                if (Deaf) {
+                    if (!Blind)
+                        pline("%s rubs %s hands with enraged delight!",
+                              noit_Monnam(grd), mhis(grd));
+                } else {
                     verbalize("So be it, rogue!");
+                }
                 grd->mpeaceful = 0;
                 return -1;
             }
@@ -767,7 +823,7 @@ nextpos:
     else
         ny += dy;
 
-    while ((typ = (crm = &levl[nx][ny])->typ) != 0) {
+    while ((typ = (crm = &levl[nx][ny])->typ) != STONE) {
         /* in view of the above we must have IS_WALL(typ) or typ == POOL */
         /* must be a wall here */
         if (isok(nx + nx - x, ny + ny - y) && !IS_POOL(typ)
