@@ -167,7 +167,7 @@ int portdebug;
         if (vk == 0xBF)
             ch = M('?');
         else
-            ch = M(tolower(keycode));
+            ch = M(tolower((uchar) keycode));
     }
     /* Attempt to work better with international keyboards. */
     else {
