@@ -1,4 +1,4 @@
-/* NetHack 3.6	gnbind.c	$NHDT-Date: 1433806614 2015/06/08 23:36:54 $  $NHDT-Branch: master $:$NHDT-Revision: 1.32 $ */
+/* NetHack 3.6	gnbind.c	$NHDT-Date: 1450453305 2015/12/18 15:41:45 $  $NHDT-Branch: NetHack-3.6.0 $:$NHDT-Revision: 1.33 $ */
 /* Copyright (C) 1998 by Erik Andersen <andersee@debian.org> */
 /* NetHack may be freely redistributed.  See license for details. */
 
@@ -48,13 +48,8 @@ struct window_procs Gnome_procs = {
     /* other defs that really should go away (they're tty specific) */
     gnome_start_screen, gnome_end_screen, gnome_outrip,
     genl_preference_update, genl_getmsghistory, genl_putmsghistory,
-#ifdef STATUS_VIA_WINDOWPORT
     genl_status_init, genl_status_finish, genl_status_enablefield,
     genl_status_update,
-#ifdef STATUS_HILITES
-    genl_status_threshold,
-#endif
-#endif
     genl_can_suspend_yes,
 };
 
@@ -1182,7 +1177,7 @@ gnome_outrip(winid wid, int how, time_t when)
     Strcat(ripString, buf);
 
     /* Put together death description */
-    formatkiller(buf, sizeof buf, how);
+    formatkiller(buf, sizeof buf, how, FALSE);
 
     /* Put death type on stone */
     Strcat(ripString, buf);
