@@ -9,7 +9,7 @@
 
 #define MSG_WRAP_TEXT
 
-#define MSG_VISIBLE_LINES max(iflags.wc_vary_msgcount, 2)
+#define MSG_VISIBLE_LINES max(iflags.wc_vary_msgcount, 1)
 #define MAX_MSG_LINES 128
 #define MSG_LINES (int) min(iflags.msg_history, MAX_MSG_LINES)
 #define MAXWINDOWTEXT TBUFSZ
@@ -41,6 +41,8 @@ typedef struct mswin_nethack_message_window {
     int yMax;   /* maximum vertical scrolling position */
     int xPage;  /* page size of horizontal scroll bar */
 } NHMessageWindow, *PNHMessageWindow;
+#define LINE_PADDING_LEFT(data)  (data->xChar * (2 - data->xPos))
+#define LINE_PADDING_RIGHT(data)  (0)
 
 static TCHAR szMessageWindowClass[] = TEXT("MSNHMessageWndClass");
 LRESULT CALLBACK NHMessageWndProc(HWND, UINT, WPARAM, LPARAM);
@@ -108,6 +110,8 @@ mswin_init_message_window()
     /* Set window caption */
     SetWindowText(ret, "Messages");
 
+    mswin_apply_window_style(ret);
+
     return ret;
 }
 
@@ -243,6 +247,25 @@ onMSNHCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
     data = (PNHMessageWindow) GetWindowLongPtr(hWnd, GWLP_USERDATA);
     switch (wParam) {
     case MSNH_MSG_PUTSTR: {
+        /* Add the passed in message to the existing text.  Support the
+         * adding of text that ends in newline.  A newline in text
+         * will force any subsequent text that is added to be added on
+         * a new output line.
+         *
+         * TODO: Text can be added with newlines occurring within the text not
+         *       just at the end.  As currently implemented, this can cause
+         *       the text to be rendered such that the text following the
+         *       newline is rendered on a new line.  This can cause a poor
+         *       user experience when the user has set only a single text line
+         *       for the message window.  In this case, the user will not see
+         *       any line other then the last line of text and the --MORE--
+         *       message thus missing any text that appears before the last
+         *       embedded newline.  This does not meet the requirements of the
+         *       message window.
+         *       This code should be changed to do the right thing and split
+         *       the text so that only lines that end in newlines are added to
+         *       the stored window text.
+         */
         PMSNHMsgPutstr msg_data = (PMSNHMsgPutstr) lParam;
         SCROLLINFO si;
         char *p;
@@ -269,10 +292,17 @@ onMSNHCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
                 /* check for "--more--" */
                 if (!data->nevermore && more_prompt_check(hWnd)) {
                     int okkey = 0;
-                    int chop;
+                    char tmptext[MAXWINDOWTEXT + 1];
+
                     // @@@ Ok respnses
 
-                    /* append more prompt and inticate the update */
+                    /* save original text */
+                    strcpy(tmptext, data->window_text[MSG_LINES - 1].text);
+
+                    /* text could end in newline so strip it */
+                    strip_newline(data->window_text[MSG_LINES - 1].text);
+
+                    /* append more prompt and indicate the update */
                     strncat(
                         data->window_text[MSG_LINES - 1].text, MORE,
                         MAXWINDOWTEXT
@@ -299,16 +329,15 @@ onMSNHCommand(HWND hWnd, WPARAM wParam, LPARAM lParam)
                         }
                     }
 
-                    /* erase the "--more--" prompt */
-                    chop = strlen(data->window_text[MSG_LINES - 1].text)
-                           - strlen(MORE);
-                    data->window_text[MSG_LINES - 1].text[chop] = '\0';
+                    /* restore original text */
+                    strcpy(data->window_text[MSG_LINES - 1].text, tmptext);
+
                     data->lines_not_seen = 0;
                 }
 
                 /* check if the string is empty */
                 for (p = data->window_text[MSG_LINES - 1].text;
-                     *p && isspace(*p); p++)
+                     *p && isspace((uchar) *p); p++)
                     ;
 
                 if (*p) {
@@ -572,7 +601,7 @@ onPaint(HWND hWnd)
     PNHMessageWindow data;
     RECT client_rt, draw_rt;
     int FirstLine, LastLine;
-    int i, x, y;
+    int i, y;
     HGDIOBJ oldFont;
     TCHAR wbuf[MAXWINDOWTEXT + 2];
     size_t wlen;
@@ -599,10 +628,10 @@ onPaint(HWND hWnd)
                     - (client_rt.bottom - ps.rcPaint.bottom) / data->yChar);
         y = min(ps.rcPaint.bottom, client_rt.bottom);
         for (i = LastLine; i >= FirstLine; i--) {
-            x = data->xChar * (2 - data->xPos);
+            char tmptext[MAXWINDOWTEXT + 1];
 
-            draw_rt.left = x;
-            draw_rt.right = client_rt.right;
+            draw_rt.left = LINE_PADDING_LEFT(data);
+            draw_rt.right = client_rt.right - LINE_PADDING_RIGHT(data);
             draw_rt.top = y - data->yChar;
             draw_rt.bottom = y;
 
@@ -610,8 +639,10 @@ onPaint(HWND hWnd)
                 hdc, mswin_get_font(NHW_MESSAGE, data->window_text[i].attr,
                                     hdc, FALSE));
 
-            /* convert to UNICODE */
-            NH_A2W(data->window_text[i].text, wbuf, sizeof(wbuf));
+            /* convert to UNICODE stripping newline */
+            strcpy(tmptext, data->window_text[i].text);
+            strip_newline(tmptext);
+            NH_A2W(tmptext, wbuf, sizeof(wbuf));
             wlen = _tcslen(wbuf);
             setMsgTextColor(hdc, i < (MSG_LINES - data->lines_last_turn));
 #ifdef MSG_WRAP_TEXT
@@ -763,6 +794,10 @@ can_append_text(HWND hWnd, int attr, const char *text)
     if (data->window_text[MSG_LINES - 1].attr != attr)
         return FALSE;
 
+    /* cannot append if current line ends in newline */
+    if (str_end_is(data->window_text[MSG_LINES - 1].text, "\n"))
+        return FALSE;
+
     /* check if the maximum string langth will be exceeded */
     if (strlen(data->window_text[MSG_LINES - 1].text) + 2
             + /* space characters */
@@ -770,10 +805,11 @@ can_append_text(HWND hWnd, int attr, const char *text)
         >= MAXWINDOWTEXT)
         return FALSE;
 
-    /* check if the text is goinf to fin into a single line */
+    /* check if the text is going to fit into a single line */
     strcpy(tmptext, data->window_text[MSG_LINES - 1].text);
     strcat(tmptext, "  ");
     strcat(tmptext, text);
+    strip_newline(tmptext);
     strcat(tmptext, MORE);
 
     hdc = GetDC(hWnd);
@@ -782,6 +818,8 @@ can_append_text(HWND hWnd, int attr, const char *text)
         mswin_get_font(NHW_MESSAGE, data->window_text[MSG_LINES - 1].attr,
                        hdc, FALSE));
     GetClientRect(hWnd, &draw_rt);
+    draw_rt.left += LINE_PADDING_LEFT(data);
+    draw_rt.right -= LINE_PADDING_RIGHT(data);
     draw_rt.bottom = draw_rt.top; /* we only need width for the DrawText */
     DrawText(hdc, tmptext, strlen(tmptext), &draw_rt,
              DT_NOPREFIX | DT_WORDBREAK | DT_CALCRECT);
@@ -826,14 +864,17 @@ more_prompt_check(HWND hWnd)
         SelectObject(hdc, mswin_get_font(NHW_MESSAGE, ATR_NONE, hdc, FALSE));
     for (i = 0; i < data->lines_not_seen; i++) {
         /* we only need width for the DrawText */
-        SetRect(&draw_rt, client_rt.left, client_rt.top, client_rt.right,
-                client_rt.top);
+        SetRect(&draw_rt,
+            client_rt.left + LINE_PADDING_LEFT(data), client_rt.top,
+            client_rt.right - LINE_PADDING_RIGHT(data), client_rt.top);
         SelectObject(hdc,
                      mswin_get_font(NHW_MESSAGE,
                                     data->window_text[MSG_LINES - i - 1].attr,
                                     hdc, FALSE));
 
         strcpy(tmptext, data->window_text[MSG_LINES - i - 1].text);
+        strip_newline(tmptext);
+
         if (i == 0)
             strcat(tmptext, MORE);
 
